@@ -16,6 +16,7 @@
 #define Bytes_val String_val
 #endif
 
+#include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libavformat/avio.h>
 #include <libavutil/audio_fifo.h>
@@ -67,6 +68,7 @@ typedef struct av_t {
 
   // output
   int header_written;
+  int (*write_frame)(AVFormatContext *, AVPacket *);
   int custom_io;
 } av_t;
 
@@ -80,7 +82,6 @@ static inline av_t *Av_val(value v) {
 
 /***** Stream handler *****/
 
-#define StreamAv_val(v) Av_val(Field(v, 0))
 #define StreamIndex_val(v) Int_val(Field(v, 1))
 
 /**** Media Type ****/
@@ -103,6 +104,9 @@ static void free_stream(stream_t *stream) {
 }
 
 static void close_av(av_t *av) {
+  if (av->closed)
+    return;
+
   caml_release_runtime_system();
 
   if (av->format_context) {
@@ -183,8 +187,9 @@ CAMLprim value ocaml_av_get_streams(value _av, value _media_type) {
 
 CAMLprim value ocaml_av_get_stream_codec_parameters(value _stream) {
   CAMLparam1(_stream);
-  CAMLlocal1(ans);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal2(ans, _av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   value_of_codec_parameters_copy(av->format_context->streams[index]->codecpar,
@@ -195,8 +200,9 @@ CAMLprim value ocaml_av_get_stream_codec_parameters(value _stream) {
 
 CAMLprim value ocaml_av_get_stream_time_base(value _stream) {
   CAMLparam1(_stream);
-  CAMLlocal1(ans);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal2(ans, _av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   value_of_rational(&av->format_context->streams[index]->time_base, &ans);
@@ -206,7 +212,9 @@ CAMLprim value ocaml_av_get_stream_time_base(value _stream) {
 
 CAMLprim value ocaml_av_set_stream_time_base(value _stream, value _time_base) {
   CAMLparam2(_stream, _time_base);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal1(_av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   av->format_context->streams[index]->time_base = rational_of_value(_time_base);
@@ -216,7 +224,9 @@ CAMLprim value ocaml_av_set_stream_time_base(value _stream, value _time_base) {
 
 CAMLprim value ocaml_av_get_stream_frame_size(value _stream) {
   CAMLparam1(_stream);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal1(_av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   CAMLreturn(Val_int(av->streams[index]->codec_context->frame_size));
@@ -224,8 +234,9 @@ CAMLprim value ocaml_av_get_stream_frame_size(value _stream) {
 
 CAMLprim value ocaml_av_get_stream_pixel_aspect(value _stream) {
   CAMLparam1(_stream);
-  CAMLlocal2(ans, ret);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal3(ans, ret, _av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
   const AVRational pixel_aspect =
       av->format_context->streams[index]->sample_aspect_ratio;
@@ -1371,7 +1382,8 @@ ocaml_av_output_format_get_subtitle_codec_id(value _output_format) {
 
 static av_t *open_output(avioformat_const AVOutputFormat *format,
                          char *file_name, AVIOContext *avio_context,
-                         value _interrupt, AVDictionary **options) {
+                         value _interrupt, int interleaved,
+                         AVDictionary **options) {
   int ret;
   AVIOInterruptCB interrupt_cb = {ocaml_av_interrupt_callback, NULL};
   AVIOInterruptCB *interrupt_cb_ptr = NULL;
@@ -1386,6 +1398,12 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
 
   av->closed = 0;
 
+  if (interleaved) {
+    av->write_frame = &av_interleaved_write_frame;
+  } else {
+    av->write_frame = &av_write_frame;
+  }
+
   if (_interrupt != Val_none) {
     av->interrupt_cb = Some_val(_interrupt);
     caml_register_generational_global_root(&av->interrupt_cb);
@@ -1485,7 +1503,8 @@ static av_t *open_output(avioformat_const AVOutputFormat *format,
 }
 
 CAMLprim value ocaml_av_open_output(value _interrupt, value _format,
-                                    value _filename, value _opts) {
+                                    value _filename, value _interleaved,
+                                    value _opts) {
   CAMLparam3(_interrupt, _filename, _opts);
   CAMLlocal3(ans, ret, unused);
   char *filename =
@@ -1511,7 +1530,8 @@ CAMLprim value ocaml_av_open_output(value _interrupt, value _format,
     format = OutputFormat_val(Some_val(_format));
 
   // open output file
-  av_t *av = open_output(format, filename, NULL, _interrupt, &options);
+  av_t *av = open_output(format, filename, NULL, _interrupt,
+                         Bool_val(_interleaved), &options);
 
   // Return unused keys
   count = av_dict_count(options);
@@ -1536,7 +1556,8 @@ CAMLprim value ocaml_av_open_output(value _interrupt, value _format,
   CAMLreturn(ret);
 }
 
-CAMLprim value ocaml_av_open_output_format(value _format, value _opts) {
+CAMLprim value ocaml_av_open_output_format(value _format, value _interleaved,
+                                           value _opts) {
   CAMLparam2(_format, _opts);
   CAMLlocal3(ans, ret, unused);
   AVDictionary *options = NULL;
@@ -1558,7 +1579,8 @@ CAMLprim value ocaml_av_open_output_format(value _format, value _opts) {
   avioformat_const AVOutputFormat *format = OutputFormat_val(_format);
 
   // open output format
-  av_t *av = open_output(format, NULL, NULL, Val_none, &options);
+  av_t *av = open_output(format, NULL, NULL, Val_none, Bool_val(_interleaved),
+                         &options);
 
   // Return unused keys
   count = av_dict_count(options);
@@ -1584,7 +1606,7 @@ CAMLprim value ocaml_av_open_output_format(value _format, value _opts) {
 }
 
 CAMLprim value ocaml_av_open_output_stream(value _format, value _avio,
-                                           value _opts) {
+                                           value _interleaved, value _opts) {
   CAMLparam3(_format, _avio, _opts);
   CAMLlocal3(ans, ret, unused);
   avioformat_const AVOutputFormat *format = OutputFormat_val(_format);
@@ -1606,7 +1628,8 @@ CAMLprim value ocaml_av_open_output_stream(value _format, value _avio,
   }
 
   // open output format
-  av_t *av = open_output(format, NULL, avio->avio_context, Val_none, &options);
+  av_t *av = open_output(format, NULL, avio->avio_context, Val_none,
+                         Bool_val(_interleaved), &options);
 
   // Return unused keys
   count = av_dict_count(options);
@@ -1985,7 +2008,9 @@ CAMLprim value ocaml_av_new_data_stream(value _av, value _codec_id,
 CAMLprim value ocaml_av_write_stream_packet(value _stream, value _time_base,
                                             value _packet) {
   CAMLparam3(_stream, _time_base, _packet);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal1(_av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int ret = 0, stream_index = StreamIndex_val(_stream);
   AVPacket *packet = Packet_val(_packet);
   AVStream *avstream = av->format_context->streams[stream_index];
@@ -2015,7 +2040,7 @@ CAMLprim value ocaml_av_write_stream_packet(value _stream, value _time_base,
   av_packet_rescale_ts(packet, rational_of_value(_time_base),
                        avstream->time_base);
 
-  ret = av_interleaved_write_frame(av->format_context, packet);
+  ret = av->write_frame(av->format_context, packet);
   av->streams[stream_index]->was_keyframe = packet->flags & AV_PKT_FLAG_KEY;
 
   caml_acquire_runtime_system();
@@ -2134,7 +2159,7 @@ static void write_frame(av_t *av, int stream_index, AVCodecContext *enc_ctx,
     packet->pos = -1;
     av_packet_rescale_ts(packet, enc_ctx->time_base, avstream->time_base);
 
-    ret = av_interleaved_write_frame(av->format_context, packet);
+    ret = av->write_frame(av->format_context, packet);
   }
 
   if (hw_frame)
@@ -2233,7 +2258,7 @@ static void write_subtitle_frame(av_t *av, int stream_index,
   packet->pos = -1;
 
   caml_release_runtime_system();
-  err = av_interleaved_write_frame(av->format_context, packet);
+  err = av->write_frame(av->format_context, packet);
   caml_acquire_runtime_system();
 
   av_packet_free(&packet);
@@ -2244,7 +2269,9 @@ static void write_subtitle_frame(av_t *av, int stream_index,
 
 CAMLprim value ocaml_av_write_stream_frame(value _stream, value _frame) {
   CAMLparam2(_stream, _frame);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal1(_av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   if (!av->streams)
@@ -2272,7 +2299,7 @@ CAMLprim value ocaml_av_flush(value _av) {
     CAMLreturn(Val_unit);
 
   caml_release_runtime_system();
-  ret = av_interleaved_write_frame(av->format_context, NULL);
+  ret = av->write_frame(av->format_context, NULL);
   if (ret >= 0 && av->format_context->pb)
     avio_flush(av->format_context->pb);
   caml_acquire_runtime_system();
@@ -2285,7 +2312,9 @@ CAMLprim value ocaml_av_flush(value _av) {
 
 CAMLprim value ocaml_av_was_keyframe(value _stream) {
   CAMLparam1(_stream);
-  av_t *av = StreamAv_val(_stream);
+  CAMLlocal1(_av);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   if (!av->streams)
@@ -2392,10 +2421,11 @@ uint8_t *ocaml_av_ff_nal_unit_extract_rbsp(const uint8_t *src, uint32_t src_len,
 // This from libavformat/hlsenc.c
 CAMLprim value ocaml_av_codec_attr(value _stream) {
   CAMLparam1(_stream);
-  CAMLlocal2(ans, _attr);
+  CAMLlocal3(ans, _attr, _av);
 
   char attr[32];
-  av_t *av = StreamAv_val(_stream);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   if (!av->format_context || !av->format_context->streams)
@@ -2413,7 +2443,7 @@ CAMLprim value ocaml_av_codec_attr(value _stream) {
       snprintf(attr, sizeof(attr), "avc1.%02x%02x%02x", data[5], data[6],
                data[7]);
     } else {
-      snprintf(attr, sizeof(attr), "avc1");
+      goto fail;
     }
   } else if (stream->codecpar->codec_id == AV_CODEC_ID_FLAC) {
     snprintf(attr, sizeof(attr), "fLaC");
@@ -2445,10 +2475,10 @@ CAMLprim value ocaml_av_codec_attr(value _stream) {
         rbsp_buf =
             ocaml_av_ff_nal_unit_extract_rbsp(data, remain_size, &rbsp_size, 0);
         if (!rbsp_buf)
-          break;
+          goto fail;
         if (rbsp_size < 13) {
           av_freep(&rbsp_buf);
-          break;
+          goto fail;
         }
         /* skip sps_video_parameter_set_id   u(4),
          *      sps_max_sub_layers_minus1    u(3),
@@ -2457,7 +2487,7 @@ CAMLprim value ocaml_av_codec_attr(value _stream) {
         /* skip 8 + 8 + 32 + 4 + 43 + 1 bit */
         level = rbsp_buf[12];
         av_freep(&rbsp_buf);
-        break;
+        goto fail;
       }
       data++;
     }
@@ -2473,14 +2503,16 @@ CAMLprim value ocaml_av_codec_attr(value _stream) {
   } else if (stream->codecpar->codec_id == AV_CODEC_ID_MP3) {
     snprintf(attr, sizeof(attr), "mp4a.40.34");
   } else if (stream->codecpar->codec_id == AV_CODEC_ID_AAC) {
-    /* TODO : For HE-AAC, HE-AACv2, the last digit needs to be set to 5 and 29
-     * respectively */
-    snprintf(attr, sizeof(attr), "mp4a.40.2");
+    if (stream->codecpar->profile != FF_PROFILE_UNKNOWN)
+      snprintf(attr, sizeof(attr), "mp4a.40.%d", stream->codecpar->profile + 1);
+    else
+      goto fail;
   } else if (stream->codecpar->codec_id == AV_CODEC_ID_AC3) {
     snprintf(attr, sizeof(attr), "ac-3");
   } else if (stream->codecpar->codec_id == AV_CODEC_ID_EAC3) {
     snprintf(attr, sizeof(attr), "ec-3");
   } else {
+  fail:
     CAMLreturn(Val_none);
   }
 
@@ -2494,9 +2526,10 @@ CAMLprim value ocaml_av_codec_attr(value _stream) {
 
 CAMLprim value ocaml_av_stream_bitrate(value _stream) {
   CAMLparam1(_stream);
-  CAMLlocal1(ans);
+  CAMLlocal2(ans, _av);
 
-  av_t *av = StreamAv_val(_stream);
+  _av = Field(_stream, 0);
+  av_t *av = Av_val(_av);
   int index = StreamIndex_val(_stream);
 
   if (!av->format_context || !av->format_context->streams)
