@@ -66,6 +66,25 @@ module Packet : sig
   (** Packet flags *)
   type flag = [ `Keyframe | `Corrupt | `Discard | `Trusted | `Disposable ]
 
+  type replaygain = {
+    track_gain : int;
+    track_peak : int;
+    album_gain : int;
+    album_peak : int;
+  }
+
+  (** Packet side_data (incomplete) *)
+  type side_data =
+    [ `Replaygain of replaygain
+    | `Strings_metadata of (string * string) list
+    | `Metadata_update of (string * string) list ]
+
+  (** Add a side data to a packet. *)
+  val add_side_data : 'media t -> side_data -> unit
+
+  (** Return a packet side_data. *)
+  val side_data : 'media t -> side_data list
+
   (** Return a fresh packet refereing the same data. *)
   val dup : 'media t -> 'media t
 
@@ -123,6 +142,12 @@ module Packet : sig
 
   (** Same as {!Avcodec.Packet.parse_data} with bytes array. *)
   val parse_bytes : 'media parser -> ('media t -> unit) -> bytes -> int -> unit
+
+  (** Advanced users: create a packet with the given data. *)
+  val create : string -> 'media t
+
+  (** Advanced users: return the packet's content. *)
+  val content : 'media t -> string
 end
 
 (** Audio codecs. *)
@@ -434,7 +459,23 @@ module Subtitle : sig
   val get_params_id : subtitle params -> id
 end
 
-(* This also includes the fake codecs. *)
+module Unknown : sig
+  type 'mode t = ([ `Data ], 'mode) codec
+
+  (* Unkown codecs seem used for data mostly. *)
+  type id = Codec_id.unknown
+
+  (** List of all unknown codec IDs. *)
+  val codec_ids : Codec_id.unknown list
+
+  (** Return the name of the codec. *)
+  val string_of_id : id -> string
+
+  (** Return the id of the codec params. *)
+  val get_params_id : [ `Data ] params -> id
+end
+
+(* This includes all the codec. *)
 type id = Codec_id.codec_id
 
 val string_of_id : id -> string
