@@ -1,3 +1,5 @@
+#define CAML_NAME_SPACE 1
+
 #include <caml/alloc.h>
 #include <caml/bigarray.h>
 #include <caml/callback.h>
@@ -15,11 +17,8 @@
 #include "hw_config_method_stubs.h"
 #include "media_types_stubs.h"
 
-#include "config.h"
-
-#ifdef HAS_BSF_H
+#include <libavutil/replaygain.h>
 #include <libavcodec/bsf.h>
-#endif
 
 #ifndef AV_PKT_FLAG_DISPOSABLE
 #define AV_PKT_FLAG_DISPOSABLE 0x0010
@@ -47,6 +46,11 @@ static value value_of_subtitle_codec_id(enum AVCodecID id) {
   return ret;
 }
 
+static value value_of_unknown_codec_id(enum AVCodecID id) {
+  value ret = Val_UnknownCodecID(id);
+  return ret;
+}
+
 CAMLprim value ocaml_avcodec_flag_qscale(value unit) {
   CAMLparam0();
   CAMLreturn(Val_int(AV_CODEC_FLAG_QSCALE));
@@ -155,6 +159,150 @@ value value_of_ffmpeg_packet(AVPacket *packet) {
   return ret;
 }
 
+CAMLprim value ocaml_avcodec_create_packet(value _data) {
+  CAMLparam1(_data);
+  AVPacket *packet;
+  int len = caml_string_length(_data);
+
+  packet = av_packet_alloc();
+  if (!packet)
+    caml_raise_out_of_memory();
+
+  int err = av_new_packet(packet, len);
+  if (err != 0) {
+    av_freep(packet);
+    ocaml_avutil_raise_error(err);
+  }
+
+  memcpy(packet->data, String_val(_data), len);
+
+  CAMLreturn(value_of_ffmpeg_packet(packet));
+}
+
+CAMLprim value ocaml_avcodec_packet_content(value _packet) {
+  CAMLparam1(_packet);
+  AVPacket *packet = Packet_val(_packet);
+  CAMLreturn(caml_alloc_initialized_string(packet->size, (char *)packet->data));
+}
+
+CAMLprim value ocaml_avcodec_packet_add_side_data(value _packet,
+                                                  value _side_data) {
+  CAMLparam2(_packet, _side_data);
+  AVPacket *packet = Packet_val(_packet);
+  enum AVPacketSideDataType type;
+  uint8_t *data;
+  AVReplayGain *replay_gain;
+  size_t len;
+
+  switch (Field(_side_data, 0)) {
+  case PVV_Metadata_update:
+    type = AV_PKT_DATA_METADATA_UPDATE;
+    break;
+  case PVV_Strings_metadata:
+    type = AV_PKT_DATA_STRINGS_METADATA;
+    break;
+  case PVV_Replaygain:
+    type = AV_PKT_DATA_REPLAYGAIN;
+    break;
+  default:
+    Fail("Invalid value");
+  }
+
+  switch (type) {
+  case AV_PKT_DATA_METADATA_UPDATE:
+  case AV_PKT_DATA_STRINGS_METADATA:
+    len = caml_string_length(Field(_side_data, 1));
+    data = av_malloc(len);
+    if (!data)
+      caml_raise_out_of_memory();
+    memcpy(data, String_val(Field(_side_data, 1)), len);
+    av_packet_add_side_data(packet, type, data, len);
+    break;
+  case AV_PKT_DATA_REPLAYGAIN:
+    len = sizeof(AVReplayGain);
+    data = av_malloc(len);
+    if (!data)
+      caml_raise_out_of_memory();
+    replay_gain = (AVReplayGain *)data;
+    replay_gain->track_gain = Int_val(Field(Field(_side_data, 1), 0));
+    replay_gain->track_peak = Int_val(Field(Field(_side_data, 1), 1));
+    replay_gain->album_gain = Int_val(Field(Field(_side_data, 1), 2));
+    replay_gain->album_peak = Int_val(Field(Field(_side_data, 1), 3));
+    av_packet_add_side_data(packet, type, data, len);
+    break;
+  default:
+    Fail("Invalid value");
+  }
+
+  CAMLreturn(Val_unit);
+}
+
+CAMLprim value ocaml_avcodec_packet_side_data(value _packet) {
+  CAMLparam1(_packet);
+  CAMLlocal3(ret, tmp, tmp2);
+  AVPacket *packet = Packet_val(_packet);
+  int len = 0;
+  int i;
+  value type;
+  AVReplayGain *replay_gain;
+
+  for (i = 0; i < packet->side_data_elems; i++) {
+    switch (packet->side_data[i].type) {
+    case AV_PKT_DATA_METADATA_UPDATE:
+    case AV_PKT_DATA_STRINGS_METADATA:
+    case AV_PKT_DATA_REPLAYGAIN:
+      len++;
+    default:
+      break;
+    }
+  }
+
+  ret = caml_alloc_tuple(len);
+
+  for (i = 0; i < len; i++) {
+    switch (packet->side_data[i].type) {
+    case AV_PKT_DATA_METADATA_UPDATE:
+    case AV_PKT_DATA_STRINGS_METADATA:
+      type = packet->side_data[i].type == AV_PKT_DATA_METADATA_UPDATE
+                 ? PVV_Metadata_update
+                 : PVV_Strings_metadata;
+
+      tmp = caml_alloc_initialized_string(packet->side_data[i].size,
+                                          (char *)packet->side_data[i].data);
+
+      tmp2 = caml_alloc_tuple(2);
+      Store_field(tmp2, 0, type);
+      Store_field(tmp2, 1, tmp);
+
+      Store_field(ret, i, tmp2);
+      break;
+
+    case AV_PKT_DATA_REPLAYGAIN:
+      if (packet->side_data[i].size < sizeof(AVReplayGain))
+        Fail("Invalid side_data");
+      replay_gain = (AVReplayGain *)packet->side_data[i].data;
+
+      tmp = caml_alloc_tuple(4);
+      Store_field(tmp, 0, Val_int(replay_gain->track_gain));
+      Store_field(tmp, 1, Val_int(replay_gain->track_peak));
+      Store_field(tmp, 2, Val_int(replay_gain->album_gain));
+      Store_field(tmp, 3, Val_int(replay_gain->album_peak));
+
+      tmp2 = caml_alloc_tuple(2);
+      Store_field(tmp2, 0, PVV_Replaygain);
+      Store_field(tmp2, 1, tmp);
+
+      Store_field(ret, i, tmp2);
+      break;
+
+    default:
+      break;
+    }
+  }
+
+  CAMLreturn(ret);
+};
+
 CAMLprim value ocaml_avcodec_packet_dup(value _packet) {
   CAMLparam1(_packet);
   CAMLlocal1(ret);
@@ -1384,6 +1532,18 @@ CAMLprim value ocaml_avcodec_parameters_video_copy_byte(value *argv, int argn) {
                                              argv[4], argv[5], argv[7]);
 }
 
+/**** Unknown codec ID *****/
+
+CAMLprim value ocaml_avcodec_get_unknown_codec_id_name(value _codec_id) {
+  CAMLparam1(_codec_id);
+  CAMLreturn(caml_copy_string(avcodec_get_name(UnknownCodecID_val(_codec_id))));
+}
+
+CAMLprim value ocaml_avcodec_parameters_get_unknown_codec_id(value _cp) {
+  CAMLparam1(_cp);
+  CAMLreturn(value_of_unknown_codec_id(CodecParameters_val(_cp)->codec_id));
+}
+
 /**** Subtitle codec ID ****/
 
 CAMLprim value ocaml_avcodec_get_subtitle_codec_id_name(value _codec_id) {
