@@ -2,6 +2,8 @@
 #include <pthread.h>
 #include <string.h>
 
+#define CAML_NAME_SPACE 1
+
 #include <caml/alloc.h>
 #include <caml/bigarray.h>
 #include <caml/callback.h>
@@ -644,6 +646,64 @@ CAMLprim value ocaml_avutil_frame_set_pts(value _frame, value _pts) {
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_avutil_frame_duration(value _frame) {
+  CAMLparam1(_frame);
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 30, 100)
+  CAMLlocal1(ret);
+  AVFrame *frame = Frame_val(_frame);
+
+  if (frame->duration == 0)
+    CAMLreturn(Val_none);
+
+  ret = caml_alloc_tuple(1);
+  Store_field(ret, 0, caml_copy_int64(frame->duration));
+
+  CAMLreturn(ret);
+#else
+  CAMLreturn(Val_none);
+#endif
+}
+
+CAMLprim value ocaml_avutil_frame_set_duration(value _frame, value _duration) {
+  CAMLparam2(_frame, _duration);
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 30, 100)
+  AVFrame *frame = Frame_val(_frame);
+
+  if (_duration == Val_none)
+    frame->duration = 0;
+  else
+    frame->duration = Int64_val(Field(_duration, 0));
+#endif
+
+  CAMLreturn(Val_unit);
+}
+
+CAMLprim value ocaml_avutil_frame_pkt_dts(value _frame) {
+  CAMLparam1(_frame);
+  CAMLlocal1(ret);
+  AVFrame *frame = Frame_val(_frame);
+
+  if (frame->pkt_dts == AV_NOPTS_VALUE)
+    CAMLreturn(Val_none);
+
+  ret = caml_alloc_tuple(1);
+  Store_field(ret, 0, caml_copy_int64(frame->pkt_dts));
+
+  CAMLreturn(ret);
+}
+
+CAMLprim value ocaml_avutil_frame_set_pkt_dts(value _frame, value _dts) {
+  CAMLparam2(_frame, _dts);
+  AVFrame *frame = Frame_val(_frame);
+
+  if (_dts == Val_none)
+    frame->pkt_dts = AV_NOPTS_VALUE;
+  else
+    frame->pkt_dts = Int64_val(Field(_dts, 0));
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_avutil_frame_metadata(value _frame) {
   CAMLparam1(_frame);
   CAMLlocal4(ans, key, val, pair);
@@ -787,7 +847,11 @@ CAMLprim value ocaml_avutil_audio_frame_get_channels(value _frame) {
   CAMLparam1(_frame);
   AVFrame *frame = Frame_val(_frame);
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(59, 19, 100)
   CAMLreturn(Val_int(frame->channels));
+#else
+  CAMLreturn(Val_int(frame->ch_layout.nb_channels));
+#endif
 }
 
 CAMLprim value ocaml_avutil_audio_frame_get_channel_layout(value _frame) {
@@ -1283,11 +1347,11 @@ CAMLprim value ocaml_avutil_av_opt_iter(value _cursor, value _class) {
     Store_field(_spec, 0, _tmp);
 
     _tmp = caml_alloc_tuple(1);
-    Store_field(_tmp, 0, Val_int(option->min));
+    Store_field(_tmp, 0, Val_int((int)option->min));
     Store_field(_spec, 1, _tmp);
 
     _tmp = caml_alloc_tuple(1);
-    Store_field(_tmp, 0, Val_int(option->max));
+    Store_field(_tmp, 0, Val_int((int)option->max));
     Store_field(_spec, 2, _tmp);
     break;
 
@@ -1314,7 +1378,7 @@ CAMLprim value ocaml_avutil_av_opt_iter(value _cursor, value _class) {
     else if (option->min >= INT64_MAX)
       Store_field(_tmp, 0, caml_copy_int64(INT64_MAX));
     else
-      Store_field(_tmp, 0, caml_copy_int64(option->min));
+      Store_field(_tmp, 0, caml_copy_int64((int64_t)option->min));
 
     Store_field(_spec, 1, _tmp);
 
@@ -1325,7 +1389,7 @@ CAMLprim value ocaml_avutil_av_opt_iter(value _cursor, value _class) {
     else if (option->max >= INT64_MAX)
       Store_field(_tmp, 0, caml_copy_int64(INT64_MAX));
     else
-      Store_field(_tmp, 0, caml_copy_int64(option->max));
+      Store_field(_tmp, 0, caml_copy_int64((int64_t)option->max));
 
     Store_field(_spec, 2, _tmp);
     break;
